@@ -96,13 +96,12 @@ The above would for example lower-case the account 'Assets:Accounts'.
 You can of course do more complex stuff like this or set $VISUAL (or $EDITOR)
 to a fancy script that can modify hledger transactions.
 
-Written by Yann Büchau, source code is at https://gitlab.com/nobodyinperson/hledger-utils
-
-
 🐛 Debugging
 -------------
 
     HLEDGER_EDIT_LOGLEVEL=DEBUG hledger edit ...
+
+version {version}, written by Yann Büchau, source code is at https://gitlab.com/nobodyinperson/hledger-utils
 """,
     formatter_class=argparse.RawDescriptionHelpFormatter,
 )
@@ -112,6 +111,9 @@ parser.add_argument(
     help="hledger executable to use. "
     "Defaults to the parent process if it looks like hledger.",
 )
+parser.add_argument(
+    "-V", "--version", action="version", version=f"%(prog)s {__version__}"
+)
 
 hledger_parser = argparse.ArgumentParser()
 # might later want to catch usage of those options
