@@ -36,6 +36,7 @@ import matplotlib.dates
 from cycler import cycler
 import psutil
 import numpy as np
+import scipy.stats
 
 import rich
 from rich import print
@@ -130,6 +131,38 @@ def regex(s):
     return pattern
 
 
+def regex_with_range(s):
+    if m := re.search(
+        r"^(?P<pattern>\S+)"
+        r"(?:\s+\[(?P<range>(?P<start>[^\]:]*):(?P<end>[^\]:]*))(?::(?P<interval>\d*))?\])?"
+        r"$",
+        s,
+    ):
+        d = m.groupdict()
+        try:
+            pattern = re.compile(d["pattern"])
+        except Exception as e:
+            raise argparse.ArgumentTypeError(
+                f"{d['pattern']!r} is not a valid regular expression: {e}"
+            )
+        try:
+            timerange = slice(
+                None if pd.isna(t := pd.to_datetime(d["start"])) else t,
+                None if pd.isna(t := pd.to_datetime(d["end"])) else t,
+            )
+        except Exception as e:
+            raise argparse.ArgumentTypeError(
+                f"{d['range']!r} is not a valid time range (error: {e}).\n"
+                f"Format example: --trend '^costs:food  [2022-01-01:2022-05-31:7]', not {s!r}"
+            )
+        interval = handle_exception(None, int, d["interval"])
+        return pattern, timerange, interval
+    else:
+        raise argparse.ArgumentTypeError(
+            f"Format example: '^costs:food  [2022-01-01:2022-05-31:7]', not {s!r}"
+        )
+
+
 def str_with_index_list(s):
     return [
         (m, handle_exception(None, int, i))
@@ -353,11 +386,28 @@ parser_modify_group.add_argument(
     type=regex,
     default=[],
 )
+parser_modify_group.add_argument(
+    "--trend",
+    metavar="REGEX [STARTDATE:ENDDATE:DAYS]",
+    help="Fit a linear trend line to columns matching the REGEX, "
+    "optionally only using a specified date range as input. "
+    "Example: '^costs:food [2022-01-01:2022-05-31:7]' to fit a trend "
+    "to the first half of 2022 and show the 7-day slope in the legend. "
+    "Whitespace in the REGEX is not allowed as a space is needed to separate it from the timerange. "
+    "You may use a dot (.) instead in the pattern. "
+    "You can leave start and/or end time out or specify 'today', but the colon (:) needs to stay. "
+    "The interval is optional and defaults to 1."
+    "Can be specified multiple times.",
+    action="append",
+    nargs="?",
+    type=regex_with_range,
+    default=[],
+)
 parser_modify_group.add_argument(
     "--mod-order",
     metavar="LIST",
     default=str_with_index_list(
-        default := "rename invert sum multiply resample mean drop only"
+        default := "rename invert sum multiply resample mean trend drop only"
     ),
     help="List of modification steps like 'sum(1),rename(2),resample(1)' "
     "specifying the application order of modifiers. "
@@ -459,6 +509,12 @@ def cli(cli_args=sys.argv[1:]):
             f"Assuming you want to --mean all columns, "
             "setting mapping to {args.mean}"
         )
+    if args.trend == [None]:
+        args.trend = [(re.compile(r".*"), slice(None, None, None), None)]
+        logger.debug(
+            f"Assuming you want to add a --trend for all columns, "
+            "setting selector to {args.trend}"
+        )
     logger.debug(f"{hledger_args = }")
 
     try:
@@ -633,6 +689,10 @@ def cli(cli_args=sys.argv[1:]):
             )
         else:
             logger.info(f"That worked, no NaNs in the amounts anymore! 🎉 ")
+            logger.info(
+                f"Consider adding {shlex.join(commodity_format_opts)} "
+                f"to your query to speed up hledger-plot."
+            )
 
     time_columns = [col for col in data if hasattr(data[col], "dt")]
     logger.debug(f"Using minimum of {time_columns = } as time")
@@ -656,7 +716,32 @@ def cli(cli_args=sys.argv[1:]):
     logger.debug(f"data = \n{data}")
 
     if args.terminal:
-        matplotlib.use("module://drawilleplot")
+        try:
+            matplotlib.use("module://drawilleplot")
+        except Exception as e:
+            logger.exception(
+                f"Couldn't use drawilleplot as matplotlib backend. "
+                f"Error: {e}"
+            )
+            logger.critical(
+                f"""
+Couldn't use drawilleplot as matplotlib backend. Error: {e}
+
+Either try to fix that or remove the --terminal option.
+
+You can try:
+
+# checking if importing drawilleplot works
+python -c 'import drawilleplot'
+
+# re-running with MPLBACKEND variable (with and/or without --terminal)
+MPLBACKEND='module://drawilleplot' hledger plot .....
+
+# on Windows one apparently needs to install windows-curses:
+pip install windows-curses
+""".strip()
+            )
+            sys.exit(1)
 
     # some sane defaults for rcParams
     plt.rcParams["legend.handlelength"] = 5
@@ -754,6 +839,48 @@ def cli(cli_args=sys.argv[1:]):
             data = data.resample(interval).sum()
         return data
 
+    def trend_handler(trenddefs, data=data):
+        for trenddef in trenddefs:
+            logger.debug(f"{trenddef = }")
+            pattern, timerange, interval = trenddef
+            if interval is None:
+                interval = next(
+                    (
+                        i
+                        for x, i in dict(
+                            daily=1,
+                            weekly=7,
+                            monthly=30,
+                            quarterly=30 * 3,
+                            yearly=360,
+                        ).items()
+                        if getattr(hledger_parsed_args, x, None)
+                    ),
+                    1,
+                )
+            if cols_to_fit := list(filter(pattern.search, data)):
+                for col in cols_to_fit:
+                    series = data.loc[timerange, col]
+                    days = series.index.map(pd.Timestamp.timestamp) / (
+                        24 * 60 * 60
+                    )
+                    fit = scipy.stats.linregress(days, series)
+                    logger.debug(f"{fit = }")
+                    data.loc[
+                        timerange,
+                        f"{col} trend ({fit.slope * interval:+f}"
+                        " / "
+                        f"{str(interval)+' ' if interval != 1 else ''}"
+                        f"day{'s' if interval!=1 else ''})",
+                    ] = (
+                        days * fit.slope + fit.intercept
+                    )
+            else:
+                logger.warning(
+                    f"--trend pattern {pattern.pattern!r} didn't match any columns."
+                )
+        return data
+
     def drop_handler(mapping, data=data):
         for pattern in mapping:
             if columns := list(filter(pattern.search, data)):
@@ -784,6 +911,7 @@ def cli(cli_args=sys.argv[1:]):
         drop=drop_handler,
         only=only_handler,
         resample=resample_handler,
+        trend=trend_handler,
     )
 
     for step, (modname, index) in enumerate(args.mod_order, start=1):
